% picard_a4xvx plots a x-vx phase space diagram for electrons and ions along
% the x axis.
%
% HG 2022-11-17

fontname = 'utopia';


run('inputpicarda4.m')
% Get species data and put it in a structure
particle = struct();
fid=fopen('inputpicarda4.m');
for ii=1:Nspecies
  % Defaults
  ppc = []; % particles per cell
  mass = [];    % mass [kg]
  charge = [];  % charge [C]
  upstreamdensity = [];  % upstream (SW) density [m^-3]
  upstreamkelvin = [];   % solar wind temperature for this species
  v0x = 0.0d3;               % solar wind velocity x component [m/s]
  v0y = 0.0d3;               % solar wind velocity y component [m/s]
  v0z = 0.0d3;               % solar wind velocity z component [m/s]
  vn  = [];               % neutral gas radial velocity component
  Qn  = [];               % neutral gas production rate [s^-1]
  Ek  = [];               % excess energy in photo-ionisation [eV]
  nu_i = [];              % ionisation frequency [s^-1]
  cometion=[];             % If 'yes', a Galand model density profile is created
                           % It should be 'no' for electrons.
  productspecies =[];        % species number for the electrons that quasi-

  theline=fgetl(fid);
  if length(theline)<5,
    theline=[theline '     '];
  end
  while ~(strcmp(theline(1:5),'%SPEC') | strcmp(theline(1:5),'%spec'))
    theline=fgetl(fid);
    if length(theline)<5,
      theline=[theline '     '];
    end
  end
  while ~(strcmp(theline(1:4),'%END') | strcmp(theline(1:4),'%end'))
    eval(theline)
    theline=fgetl(fid);
    if length(theline)<4,
      theline=[theline '    '];
    end
  end
  particle(ii).ppc = ppc;
  particle(ii).mass = mass;
  particle(ii).charge = charge;
  particle(ii).upstreamdensity = upstreamdensity;
  particle(ii).v0x = v0x;
  particle(ii).v0y = v0y;
  particle(ii).v0z = v0z;
  particle(ii).vn = vn;
  particle(ii).Qn = Qn;
  particle(ii).Ek = Ek;
  particle(ii).nu_i = nu_i;
  particle(ii).cometion = cometion;
  particle(ii).productspecies = productspecies;
end
fclose(fid);


spec = h5info('outp/parameters.h5');
clear species
for ii = 1:Nspecies
    species(ii) = h5read('outp/parameters.h5',['/' spec.Datasets(ii).Name]);
end

figure(61002)
clf
fpos = get(gcf,'pos');
set(gcf,'paperpositionmode','auto', ...
        'position',[fpos(1) fpos(2)+fpos(4)-900 1500 900])

Nr = 2;   % Number of rows
hi=[];
for ii = 0:Nr-1
  hi(   1+ii) = axes('position',[0.07 0.54-0.455*ii 0.91 0.45]);
end

figure(61020)
clf
fpos = get(gcf,'pos');
set(gcf,'paperpositionmode','auto', ...
        'position',[fpos(1) fpos(2)+fpos(4)-800 1000 800])

hj=[];
hj(1) = axes('position',[0.07 0.810 0.40 0.16]);
hj(2) = axes('position',[0.57 0.810 0.40 0.16]);
hj(3) = axes('position',[0.07 0.575 0.91 0.17]);
hj(4) = axes('position',[0.07 0.400 0.91 0.17]);
hj(5) = axes('position',[0.07 0.225 0.91 0.17]);
hj(6) = axes('position',[0.07 0.050 0.91 0.17]);


%% Load the particles
coord=struct([]); % we shall store particle data in this struct
for ii=1:Nspecies
    coord(ii).r = [];
    coord(ii).v = [];
end
coordinates.r=[];coordinates.sp=[];coordinates.v=[];

% The files that represent particles closest to the x axis are given by
% ix=[0:7], iy=[3:4], and iz=[3:4].
% $$$ for iz = 0:1
% $$$     for iy = 0:1
% $$$         for ix = 0:1
for iz = 3:4
    for iy = 3:4
        for ix = 0:7
            proc = num2str(ix + jprocs*iy + iprocs*jprocs*iz,'%5.5d');
            filename =['outp/particles/particles_p' proc '.h5'];
            pinfo=h5info(filename);
            coordinates.r = h5read(filename,[pinfo.Groups(end).Name '/r']);
            coordinates.sp = h5read(filename,[pinfo.Groups(end).Name '/species']);
            coordinates.v = h5read(filename,[pinfo.Groups(end).Name '/v']);
            for ii=1:Nspecies
                ind = coordinates.sp == ii;
                coord(ii).r = [coord(ii).r;coordinates.r(:,ind).'];
                coord(ii).v = [coord(ii).v;coordinates.v(:,ind).'];
            end
        end
    end
end
clear coordinates

%% Some grid and particle quantities
Nx=Nx_local*iprocs;
Ny=Ny_local*jprocs;
Nz=Nz_local*kprocs;
dxyz=[(xmax-xmin)/Nx (ymax-ymin)/Ny (zmax-zmin)/Nz];
xcorn = xmin + [0:Nx]*dxyz(1); x = 0.5*(xcorn(1:end-1)+xcorn(2:end));
ycorn = ymin + [0:Ny]*dxyz(2); y = 0.5*(ycorn(1:end-1)+ycorn(2:end));
zcorn = zmin + [0:Nz]*dxyz(3); z = 0.5*(zcorn(1:end-1)+zcorn(2:end));
Nprocs = iprocs*jprocs*kprocs;
dV = prod(dxyz);
if ~isfield(particle,'n2p')
    for ii=1:Nspecies
        if particle(ii).upstreamdensity > 0.0d0
            if particle(ii).ppc > 0
                particle(ii).n2p = dV*particle(ii).upstreamdensity / ...
                    particle(ii).ppc;
            else
                particle(ii).n2p = particle(1).n2p;
            end
        else
            particle(ii).n2p=particle(1).n2p*particle(1).ppc/particle(ii).ppc;
        end
    end
end


%% Testplot to see if we have loaded the right files
if 1==2
figure(8282)
subplot(2,4,1);
plot(coord(1).r(:,1),coord(1).r(:,2),'r.');xlabel('x');ylabel('y')
subplot(2,4,5);
plot(coord(1).r(:,1),coord(1).r(:,3),'r.');xlabel('x');ylabel('z')

subplot(2,4,2);
plot(coord(2).r(:,1),coord(2).r(:,2),'g.');xlabel('x');ylabel('y')
subplot(2,4,6);
plot(coord(2).r(:,1),coord(2).r(:,3),'g.');xlabel('x');ylabel('z')

subplot(2,4,3);
plot(coord(3).r(:,1),coord(3).r(:,2),'b.');xlabel('x');ylabel('y')
subplot(2,4,7);
plot(coord(3).r(:,1),coord(3).r(:,3),'b.');xlabel('x');ylabel('z')

subplot(2,4,4);
plot(coord(4).r(:,1),coord(4).r(:,2),'k.');xlabel('x');ylabel('y')
subplot(2,4,8);
plot(coord(4).r(:,1),coord(4).r(:,3),'k.');xlabel('x');ylabel('z')
end

%% Pick out a small region in the y-z plane and save x and vx for that.
y0 = 0; z0 = 0;
y1 = y0 - dxyz(2); y2 = y0 + dxyz(2);
z1 = z0 - dxyz(3); z2 = z0 + dxyz(3);
subset=struct([]);
for ii = 1:Nspecies
    ind = coord(ii).r(:,2) >= y1 & coord(ii).r(:,2)<=y2 & ...
          coord(ii).r(:,3) >= z1 & coord(ii).r(:,3)<=z2;
    subset(ii).x  = coord(ii).r(ind,1);
    subset(ii).y  = coord(ii).r(ind,2);
    subset(ii).z  = coord(ii).r(ind,3);
    subset(ii).vx = coord(ii).v(ind,1);
    subset(ii).vy = coord(ii).v(ind,2);
    subset(ii).vz = coord(ii).v(ind,3);
end

plot(hi(1),subset(1).x,subset(1).vx*1e-6,'r.', ...
     subset(4).x,subset(4).vx*1e-6,'k.')
plot(hi(2),subset(2).x,subset(2).vx*1e-3,'g.', ...
     subset(3).x,subset(3).vx*1e-3,'b.')
set(hi,'fontname',fontname,'fontsize',14)
set(hi(1),'xticklabel',[])
xlabel(hi(2),'$x$  [m]','interpreter','latex','fontname',fontname,'fontsize',18)
ylabel(hi(1),'$v_{x}$  [Mm/s]','interpreter','latex', ...
       'fontname',fontname,'fontsize',18)
ylabel(hi(2),'$v_{x}$  [km/s]','interpreter','latex', ...
       'fontname',fontname,'fontsize',18)
hleg(1)=legend(hi(1),'SW electrons','Cometary electrons');
set(hleg(1),'fontname',fontname,'fontsize',18);

hleg(2)=legend(hi(2),'SW protons','Cometary H$_{2}$O$^{+}$');
set(hleg(2),'fontname',fontname,'fontsize',18,'interpreter','latex');

%% Compute temperatures
for jj=1:Nx
    for ii = 1:Nspecies
        clear ind
        ind = subset(ii).x > xcorn(jj) & subset(ii).x <= xcorn(jj+1);
        subset(ii).n(jj) = sum(ind)*particle(ii).n2p/(dxyz(1)*(y2-y1)*(z2-z1));
        subset(ii).v0x(jj) = mean(subset(ii).vx(ind));
        subset(ii).v0y(jj) = mean(subset(ii).vy(ind));
        subset(ii).v0z(jj) = mean(subset(ii).vz(ind));
        subset(ii).kTx(jj) = particle(ii).mass * ...
            mean((subset(ii).vx(ind)-subset(ii).v0x(jj)).^2);
        subset(ii).kTy(jj) = particle(ii).mass * ...
            mean((subset(ii).vy(ind)-subset(ii).v0y(jj)).^2);
        subset(ii).kTz(jj) = particle(ii).mass * ...
            mean((subset(ii).vz(ind)-subset(ii).v0z(jj)).^2);
    end
end

for ii = 1:Nspecies
    subset(ii).kT =0.5*(subset(ii).kTx+subset(ii).kTy+subset(ii).kTz);
    subset(ii).lambda = sqrt(eps0*subset(ii).kT./(subset(ii).n*elc^2));
end

plot(hj(3),x,subset(1).n*1e-6,'b',x,subset(4).n*1e-6,'r')
plot(hj(4),x,subset(1).kTx/elc,'b',x,subset(1).kTy/elc,'g', ...
     x,subset(1).kTz/elc,'r')
plot(hj(5),x,subset(4).kTx/elc,'b',x,subset(4).kTy/elc,'g', ...
     x,subset(4).kTz/elc,'r')
plot(hj(6),x,subset(1).lambda,'b',x,subset(4).lambda,'r')
linkaxes(hj(3:6),'x')
set(hj,'fontname',fontname,'fontsize',14)
set(hj([3:5]),'xticklabel',[])

xlabel(hj(1),'v_{x}   [m/s]','fontname',fontname,'fontsize',14)
xlabel(hj(2),'v_{x}   [m/s]','fontname',fontname,'fontsize',14)
xlabel(hj(6),'x   [m]','fontname',fontname,'fontsize',14)
ylabel(hj(6),'$\lambda_\mathrm{De}$   [m]','interpreter','latex','fontname',fontname,'fontsize',14)
ylabel(hj(5),'$k_\mathrm{B}T_\mathrm{e}$   [eV]','interpreter','latex','fontname',fontname,'fontsize',14)
ylabel(hj(4),'$k_\mathrm{B}T_\mathrm{e}$   [eV]','interpreter','latex','fontname',fontname,'fontsize',14)
ylabel(hj(3),'$n_\mathrm{e}$   [cm$^{-3}$]','interpreter','latex','fontname',fontname,'fontsize',14)
hleg=legend(hj(3),'SW electrons','Cometary electrons');
set(hleg,'fontname',fontname,'fontsize',14)
hleg=legend(hj(4),'SW electr, kT_x','SW electr,kT_y','SW electr,kT_z');
set(hleg,'fontname',fontname,'fontsize',14);
hleg=legend(hj(5),'Cometary electr, kT_x','Cometary electr,kT_y','Cometary electr,kT_z');
set(hleg,'fontname',fontname,'fontsize',14)
hleg=legend(hj(6),'SW electrons','Cometary electrons');
set(hleg,'fontname',fontname,'fontsize',14)
title(hj(1),'SW electrons','fontname',fontname,'fontsize',14)
title(hj(2),'Cometary electrons','fontname',fontname,'fontsize',14)



% histograms for distribution functions
clear ind
ind = subset(1).x > -dxyz(1) & subset(1).x <= dxyz(1);
hist(hj(1),subset(1).vx(ind),50)
clear ind
ind = subset(4).x > -dxyz(1) & subset(4).x <= dxyz(1);
hist(hj(2),subset(4).vx(ind),50)
set(hj,'fontname',fontname,'fontsize',14)



if 3==4
  ccc=pwd;
  cd ~
  print -f6002 -dpng fig-x-vx.png
  cd(ccc)
end
